;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 1 test for TEB bands. Calculates the Blackbody Calibration Source 
;  (BCS) radiance.
;
;  Input:
;  1) Input file with paths to RSR, RVS, and general info sav files as well
;     as the .temp.sav file.
;
;  Output:
;  1) rad_BCS[collect,band,HAM,detector,subsample,0=RSR 1=no RSR]
;  2) rad_OBC[collect,band,HAM,detector,subsample,0=RSR 1=no RSR]
;  3) rad_p1[band,collect,0=BCS 1=SVS,0=RSR 1=no RSR]
;  4) rad_p2[band,HAM,collect,0=HAM 1=RTA 2=CAV 3=SH,4=OBC,0=RSR 1=no RSR]
;  5) rvs[band,subsample,detector,HAM,0=BCS 1=SVS]
;  6) ncollects
;  7) p -- band dependent reflectance.
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_rad, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,0
ham=initial_ham_side(inpath,header_size)

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'
restore,param_in

max_scan=100					; # of scans.

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=''
FOR i=0, a3-1 DO BEGIN
	outfile=outfile+'/'+a2[i]
ENDFOR
outfile=outfile+'/'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
print,outfile

rad_p1=dblarr(ncollects,nbands_TEB,maxdet,2,2)
rad_p2=dblarr(ncollects,nbands_TEB,nham,maxdet,5,2)
rad_p3=dblarr(ncollects,nbands_TEB,nham,maxdet,max_scan,5,2)

rvs=dblarr(nbands_TEB,maxsubsets,maxdet,nham,2)

rad_BCS=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,2)
rad_OBC_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
rad_OBC=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)

;*****************************************************************************
;  Calculate angles.
;*****************************************************************************

win_num=4

; Starting position of the collect window in samples.
MSample_offset=1064*win_num
ISample_offset=2128*win_num

Msample_max=1850+MSample_offset
Msample_min=1650+MSample_offset
Isample_max=3700+ISample_offset
Isample_min=3300+ISample_offset

; Center sample numbers.
Msample_center=0.5*(Msample_max+Msample_min)
Isample_center=0.5*(Isample_max+Isample_min)

; Bore-sight scan angles.
Theta_M=((Msample_center-33.5)*0.017785-56.063)*(!pi/180)
Theta_I=(0.5*(Isample_center-67)*0.017785-56.063)*(!pi/180)

Theta_SVS=-65.68*(!pi/180)

Theta_ref=46*(!pi/180)

; HAM angle of incidence.
Phi_perpendicular=28.6*(!pi/180)
Phi=dblarr(nbands_TEB,2)				; All bands for BCS and SVS.

FOR b=0, nbands_TEB-1 DO BEGIN
	IF b LE 1 THEN Phi[b,0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_I-Theta_ref)))*(180/!pi)
	IF b GT 1 THEN Phi[b,0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_M-Theta_ref)))*(180/!pi)
	Phi[b,1]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_SVS-Theta_ref)))*(180/!pi)
ENDFOR

;*****************************************************************************
;  Read in scan averaged temperatures for the HAM, RTA, CAV, OBC BB,
;  SH, CPFA, OMM, LWASP, MWASP, BCS, and SVS.
;*****************************************************************************

restore,outpath+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
print,outpath+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

;*****************************************************************************
;  Calculate the radiances, both integrating the RSR and L over wavelength 
;  and also the Planck radiance.
;*****************************************************************************

; Loop over BCS, rad_p1(i=0), SVS, rad_p1(i=1), HAM, rad_p2(i=0), RTA, rad_p2(i=1),
; CAV, rad_p2(i=2), SH, rad_p2(i=3), OBC, rad_p2(i=4).

restore,rsrpath

FOR col=0, ncollects-1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		rad_p1[col,b,0:detectors_TEB[b]-1,0,0]=get_avg_2(rsrf,T_bcs[col],b,detectors_TEB[b])
		rad_p1[col,b,0:detectors_TEB[b]-1,1,0]=get_avg_2(rsrf,T_svs[col],b,detectors_TEB[b])
		FOR ih=0, nham-1 DO BEGIN
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,0,0]=get_avg_2(rsrf,T_ham[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,1,0]=get_avg_2(rsrf,T_rta[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,2,0]=get_avg_2(rsrf,T_cav[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,3,0]=get_avg_2(rsrf,T_sh[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,4,0]=get_avg_2(rsrf,T_obc[ih,col],b,detectors_TEB[b])
		ENDFOR
	ENDFOR
ENDFOR

FOR col=0, ncollects-1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR id=0, detectors_TEB[b]-1 DO BEGIN
			rad_p1[col,b,id,0,1]=Planck(T_bcs[col],lambda_TEB[b])
			rad_p1[col,b,id,1,1]=Planck(T_svs[col],lambda_TEB[b])
			FOR ih=0, nham-1 DO BEGIN
				rad_p2[col,b,ih,id,0,1]=Planck(T_ham[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,1,1]=Planck(T_rta[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,2,1]=Planck(T_cav[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,3,1]=Planck(T_sh[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,4,1]=Planck(T_obc[ih,col],lambda_TEB[b])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

FOR iscan=0, max_scan-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR b=0, nbands_TEB-1 DO BEGIN
			FOR col=0, ncollects-1 DO BEGIN
				rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,0] $
					=get_avg_2(rsrf,T_ham_2[iscan,ih,col],b,detectors_TEB[b])
				rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,1] $
					=get_avg_2(rsrf,T_rta_2[iscan,ih,col],b,detectors_TEB[b])
				rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,2] $
					=get_avg_2(rsrf,T_cav_2[iscan,ih,col],b,detectors_TEB[b])
				rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,3] $
					=get_avg_2(rsrf,T_sh_2[iscan,ih,col],b,detectors_TEB[b])
				rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,4] $
					=get_avg_2(rsrf,T_obc_2[iscan,ih,col],b,detectors_TEB[b])
			ENDFOR
		ENDFOR
        ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the RVS.
;*****************************************************************************

; Restore RVS data called COEFF=dblarr(# bands,subsample,# detectors,HAM side,three fit parameters (ao,a1,a2)).
restore,rvs_in
print,rvs_in

FOR b=0, nbands_TEB-1 DO BEGIN

	; 0 for BCS and 1 for SVS.
	FOR j=0, 1 DO BEGIN
		IF band_type_TEB[b] EQ 1 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*Phi[b,j]+COEF[b+6,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*38.53+COEF[b+6,*,*,*,2]*38.53^2)
		ENDIF
		IF band_type_TEB[b] EQ 0 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*Phi[b,j]+COEF[b-2,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*38.53+COEF[b-2,*,*,*,2]*38.53^2)
		ENDIF
	ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the BCS and OBC radiances.
;*****************************************************************************

;  0 for rsr correction and 1 for no rsr correction.
FOR rsr=0, 1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR col=0, ncollects-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR ih=0, nham-1 DO BEGIN
					rad_BCS[col,b,ih,*,is,rsr]=p[b]*(rvs[b,is,*,ih,0]*rad_p1[col,b,*,0,rsr] $
						-rvs[b,is,*,ih,1]*rad_p1[col,b,*,1,rsr]) $
						+(rvs[b,is,*,ih,1]-rvs[b,is,*,ih,0]) $
						*(rad_p2[col,b,ih,*,0,rsr]-(1-p[b])*rad_p2[col,b,ih,*,1,rsr])
                                ENDFOR

			        IF ham[col] EQ 0 THEN ih=0
		      	        IF ham[col] EQ 1 THEN ih=1
			        FOR iscan=0, max_scan-1 DO BEGIN
				        rad_OBC_1[col,b,ih,0:detectors_TEB[b]-1,is,iscan]=emiss_bb $
					        *rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,4]+(1-emiss_bb) $
					        *(f_cav*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,2] $
					        +f_sh*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,3] $
					        +f_rta*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,1])

				        rad_OBC[col,b,ih,0:detectors_TEB[b]-1,is,iscan] $
					        =p[b]*(rad_OBC_1[col,b,ih,0:detectors_TEB[b]-1,is,iscan] $
					        -rvs[b,is,0:detectors_TEB[b]-1,ih,1]*rad_p1[col,b,0:detectors_TEB[b]-1,1]) $
					        +(rvs[b,is,0:detectors_TEB[b]-1,ih,1]-1) $
					        *(rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,0] $
					        -(1-p[b])*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,1])
			     	        IF ih EQ 0 THEN BEGIN
					        ih=1
					        goto,skip
				        ENDIF
				        IF ih EQ 1 THEN ih=0
				        skip:
			        ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,ncollects,rad_BCS,rad_OBC,rad_p1,rad_p2,rad_p3,rvs,p,filename=outfile

END

